@@ -407,7 +407,7 @@ void Kleo::KeyListViewItem::setKey(const GpgME::Key &key)
         }
         if (ds) {
             setForeground(i, QBrush(ds->keyForeground(key, foreground(i).color())));
-            setBackgroundColor(i, ds->keyBackground(key, backgroundColor(i)));
+            setBackground(i, QBrush(ds->keyBackground(key, background(i).color())));
             setFont(i, ds->keyFont(key, font(i)));
         }
     }
@@ -425,7 +425,7 @@ bool Kleo::KeyListViewItem::operator<(const QTreeWidgetItem &other) const
     if (other.type() != RTTI || !listView() || !listView()->columnStrategy()) {
         return QTreeWidgetItem::operator<(other);
     }
-    const auto *that = static_cast<const KeyListViewItem *>(&other);
+    const auto that = static_cast<const KeyListViewItem *>(&other);
     return listView()->columnStrategy()->compare(this->key(), that->key(), treeWidget()->sortColumn()) < 0;
 }
 
@@ -453,7 +453,7 @@ int Kleo::KeyListView::ColumnStrategy::compare(const GpgME::Key &key1, const Gpg
 
 int Kleo::KeyListView::ColumnStrategy::width(int col, const QFontMetrics &fm) const
 {
-    return fm.width(title(col)) * 2;
+    return fm.horizontalAdvance(title(col)) * 2;
 }
 
 QString Kleo::KeyListView::ColumnStrategy::toolTip(const GpgME::Key &key, int col) const
@@ -526,7 +526,8 @@ Kleo::KeyListViewItem *Kleo::KeyListView::selectedItem() const
 QList<Kleo::KeyListViewItem *> Kleo::KeyListView::selectedItems() const
 {
     QList<KeyListViewItem *> result;
-    foreach (QTreeWidgetItem *selectedItem, QTreeWidget::selectedItems()) {
+    const auto selectedItems = QTreeWidget::selectedItems();
+    for (QTreeWidgetItem *selectedItem : selectedItems) {
         if (auto *i = Kleo::lvi_cast<Kleo::KeyListViewItem>(selectedItem)) {
             result.append(i);
         }
