@@ -8,6 +8,7 @@
 #include <kleo_ui_debug.h>
 
 #include "kleo/dn.h"
+#include "models/keylist.h"
 #include "models/keylistmodel.h"
 #include "models/keylistsortfilterproxymodel.h"
 #include "models/keycache.h"
@@ -23,6 +24,8 @@
 
 #include <KLocalizedString>
 
+using namespace Kleo;
+
 Q_DECLARE_METATYPE(GpgME::Key)
 
 namespace
@@ -52,8 +55,8 @@ public:
 
     bool lessThan(const QModelIndex &left, const QModelIndex &right) const override
     {
-        const auto leftKey =  sourceModel()->data(left, Kleo::KeyListModelInterface::KeyRole).value<GpgME::Key>();
-        const auto rightKey = sourceModel()->data(right, Kleo::KeyListModelInterface::KeyRole).value<GpgME::Key>();
+        const auto leftKey =  sourceModel()->data(left, KeyList::KeyRole).value<GpgME::Key>();
+        const auto rightKey = sourceModel()->data(right, KeyList::KeyRole).value<GpgME::Key>();
         if (leftKey.isNull()) {
             return false;
         }
@@ -178,7 +181,7 @@ public:
 
     Qt::ItemFlags flags(const QModelIndex &index) const override
     {
-        Q_UNUSED(index);
+        Q_UNUSED(index)
         return Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemNeverHasChildren;
     }
 
@@ -196,7 +199,7 @@ public:
 
         if (isCustomItem(index.row())) {
             Q_ASSERT(!mFrontItems.isEmpty() || !mBackItems.isEmpty());
-            auto *ci = static_cast<CustomItem*>(index.internalPointer());
+            auto ci = static_cast<CustomItem*>(index.internalPointer());
             switch (role) {
             case Qt::DisplayRole:
                 return ci->text;
@@ -211,7 +214,7 @@ public:
             }
         }
 
-        const auto key = QSortFilterProxyModel::data(index, Kleo::KeyListModelInterface::KeyRole).value<GpgME::Key>();
+        const auto key = QSortFilterProxyModel::data(index, KeyList::KeyRole).value<GpgME::Key>();
         Q_ASSERT(!key.isNull());
         if (key.isNull()) {
             return QVariant();
@@ -293,7 +296,7 @@ public:
 
         for (int i = 0; i < proxyModel->rowCount(); ++i) {
             const auto idx = proxyModel->index(i, 0, QModelIndex());
-            const auto key = proxyModel->data(idx, Kleo::KeyListModelInterface::KeyRole).value<GpgME::Key>();
+            const auto key = proxyModel->data(idx, KeyList::KeyRole).value<GpgME::Key>();
             if (key.isNull()) {
                 // WTF?
                 continue;
@@ -391,7 +394,7 @@ void KeySelectionCombo::init()
             this, [this]() {
                     // Set useKeyCache ensures that the cache is populated
                     // so this can be a blocking call if the cache is not initialized 
-                    d->model->useKeyCache(true, d->secretOnly);
+                    d->model->useKeyCache(true, d->secretOnly ? KeyList::SecretKeysOnly : KeyList::AllKeys);
                     d->proxyModel->removeCustomItem(QStringLiteral("-libkleo-loading-keys"));
 
                     // We use the useWasEnabled state variable to decide if we should
@@ -416,7 +419,7 @@ void KeySelectionCombo::init()
     if (!d->cache->initialized()) {
         refreshKeys();
     } else {
-        d->model->useKeyCache(true, d->secretOnly);
+        d->model->useKeyCache(true, d->secretOnly ? KeyList::SecretKeysOnly : KeyList::AllKeys);
         Q_EMIT keyListingFinished();
     }
 
@@ -452,12 +455,12 @@ QString KeySelectionCombo::idFilter() const
 
 GpgME::Key Kleo::KeySelectionCombo::currentKey() const
 {
-    return currentData(Kleo::KeyListModelInterface::KeyRole).value<GpgME::Key>();
+    return currentData(KeyList::KeyRole).value<GpgME::Key>();
 }
 
 void Kleo::KeySelectionCombo::setCurrentKey(const GpgME::Key &key)
 {
-    const int idx = findData(QVariant::fromValue(key), Kleo::KeyListModelInterface::KeyRole, Qt::MatchExactly);
+    const int idx = findData(QVariant::fromValue(key), KeyList::KeyRole, Qt::MatchExactly);
     if (idx > -1) {
         setCurrentIndex(idx);
     } else {
@@ -474,16 +477,10 @@ void Kleo::KeySelectionCombo::setCurrentKey(const QString &fingerprint)
         // Already set
         return;
     }
-    for (int i = 0; i < d->proxyModel->rowCount(); ++i) {
-        const auto idx = d->proxyModel->index(i, 0, QModelIndex());
-        const auto key = d->proxyModel->data(idx, Kleo::KeyListModelInterface::KeyRole).value<GpgME::Key>();
-        if (!key.isNull() && fingerprint == QString::fromLatin1(key.primaryFingerprint())) {
-            setCurrentIndex(i);
-            setToolTip(currentData(Qt::ToolTipRole).toString());
-            return;
-        }
-    }
-    if (!d->selectPerfectIdMatch()) {
+    const int idx = findData(fingerprint, KeyList::FingerprintRole, Qt::MatchExactly);
+    if (idx > -1) {
+        setCurrentIndex(idx);
+    } else if (!d->selectPerfectIdMatch()) {
         setCurrentIndex(0);
     }
     setToolTip(currentData(Qt::ToolTipRole).toString());
