@@ -2,32 +2,9 @@
     filesystemwatcher.cpp
 
     This file is part of Kleopatra, the KDE keymanager
-    Copyright (c) 2008 Klarälvdalens Datakonsult AB
+    SPDX-FileCopyrightText: 2008 Klarälvdalens Datakonsult AB
 
-    Kleopatra is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    Kleopatra is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-
-    In addition, as a special exception, the copyright holders give
-    permission to link the code of this program with any edition of
-    the Qt library by Trolltech AS, Norway (or with modified versions
-    of Qt that use the same license as Qt), and distribute linked
-    combinations including the two.  You must obey the GNU General
-    Public License in all respects for all of the code used other than
-    Qt.  If you modify this file, you may extend this exception to
-    your version of the file, but you are not obligated to do so.  If
-    you do not wish to do so, delete this exception statement from
-    your version.
+    SPDX-License-Identifier: GPL-2.0-or-later
 */
 
 #include "filesystemwatcher.h"
@@ -62,7 +39,7 @@ public:
 
     void connectWatcher();
 
-    QFileSystemWatcher *m_watcher;
+    QFileSystemWatcher *m_watcher = nullptr;
     QTimer m_timer;
     std::set<QString> m_seenPaths;
     std::set<QString> m_cachedDirectories;
@@ -76,7 +53,7 @@ FileSystemWatcher::Private::Private(FileSystemWatcher *qq, const QStringList &pa
       m_paths(paths)
 {
     m_timer.setSingleShot(true);
-    connect(&m_timer, SIGNAL(timeout()), q, SLOT(onTimeout()));
+    connect(&m_timer, &QTimer::timeout, q, [this]() { onTimeout(); });
 }
 
 static bool is_matching(const QString &file, const QStringList &list)
@@ -202,8 +179,8 @@ void FileSystemWatcher::Private::connectWatcher()
     if (!m_watcher) {
         return;
     }
-    connect(m_watcher, SIGNAL(directoryChanged(QString)), q, SLOT(onDirectoryChanged(QString)));
-    connect(m_watcher, SIGNAL(fileChanged(QString)), q, SLOT(onFileChanged(QString)));
+    connect(m_watcher, &QFileSystemWatcher::directoryChanged, q, [this](const QString &str) { onDirectoryChanged(str); });
+    connect(m_watcher, &QFileSystemWatcher::fileChanged, q, [this](const QString &str) { onFileChanged(str); });
 }
 
 FileSystemWatcher::FileSystemWatcher(QObject *p)
@@ -299,7 +276,7 @@ void FileSystemWatcher::addPaths(const QStringList &paths)
     }
     const QStringList newPaths = paths + resolve(paths, d->m_blacklist, d->m_whitelist);
     if (!newPaths.empty()) {
-        qCDebug(LIBKLEO_LOG) << "adding\n " << newPaths.join(QStringLiteral("\n ")) << "\n/end";
+        qCDebug(LIBKLEO_LOG) << "adding\n " << newPaths.join(QLatin1String("\n ")) << "\n/end";
     }
     d->m_paths += newPaths;
     d->m_seenPaths.insert(newPaths.begin(), newPaths.end());
